@@ -11,9 +11,12 @@ import tqdm
 import pickle
 
 parser = ConfigParser()
-config_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'configuration.ini')
+config_file = os.path.join(
+    os.path.dirname(os.path.abspath(__file__)), "configuration.ini"
+)
 parser.read(config_file)
 
+
 class Sorn(object):
 
     # SORN network Initialization
@@ -22,33 +25,37 @@ class Sorn(object):
         pass
 
     """Get network variables from configuration file as class variables of SORN"""
-    
-    nu = int(parser.get('Network_Config', 'Nu'))  # Number of input units
-    ne = int(parser.get('Network_Config', 'Ne'))  # Number of excitatory units
+
+    nu = int(parser.get("Network_Config", "Nu"))  # Number of input units
+    ne = int(parser.get("Network_Config", "Ne"))  # Number of excitatory units
     ni = int(0.2 * ne)  # Number of inhibitory units in the network
-    
-    eta_stdp = float(parser.get('Network_Config', 'eta_stdp'))
-    eta_inhib = float(parser.get('Network_Config', 'eta_inhib'))
-    eta_ip = float(parser.get('Network_Config', 'eta_ip'))
-    te_max = float(parser.get('Network_Config', 'te_max'))
-    ti_max = float(parser.get('Network_Config', 'ti_max'))
-    ti_min = float(parser.get('Network_Config', 'ti_min'))
-    te_min = float(parser.get('Network_Config', 'te_min'))
-    mu_ip = float(parser.get('Network_Config', 'mu_ip'))
-    sigma_ip = float(parser.get('Network_Config', 'sigma_ip'))  # Standard deviation, variance == 0
 
-    network_type_ee = str(parser.get('Network_Config','network_type_ee'))
-    network_type_ei = str(parser.get('Network_Config','network_type_ei'))
-    network_type_ie = str(parser.get('Network_Config','network_type_ie'))
+    eta_stdp = float(parser.get("Network_Config", "eta_stdp"))
+    eta_inhib = float(parser.get("Network_Config", "eta_inhib"))
+    eta_ip = float(parser.get("Network_Config", "eta_ip"))
+    te_max = float(parser.get("Network_Config", "te_max"))
+    ti_max = float(parser.get("Network_Config", "ti_max"))
+    ti_min = float(parser.get("Network_Config", "ti_min"))
+    te_min = float(parser.get("Network_Config", "te_min"))
+    mu_ip = float(parser.get("Network_Config", "mu_ip"))
+    sigma_ip = float(
+        parser.get("Network_Config", "sigma_ip")
+    )  # Standard deviation, variance == 0
 
-    lambda_ee = int(parser.get('Network_Config','lambda_ee'))
-    lambda_ei = int(parser.get('Network_Config','lambda_ei'))
-    lambda_ie = int(parser.get('Network_Config','lambda_ie'))
+    network_type_ee = str(parser.get("Network_Config", "network_type_ee"))
+    network_type_ei = str(parser.get("Network_Config", "network_type_ei"))
+    network_type_ie = str(parser.get("Network_Config", "network_type_ie"))
+
+    lambda_ee = int(parser.get("Network_Config", "lambda_ee"))
+    lambda_ei = int(parser.get("Network_Config", "lambda_ei"))
+    lambda_ie = int(parser.get("Network_Config", "lambda_ie"))
 
     # Initialize weight matrices
 
     @staticmethod
-    def initialize_weight_matrix(network_type, synaptic_connection, self_connection, lambd_w):
+    def initialize_weight_matrix(
+        network_type, synaptic_connection, self_connection, lambd_w
+    ):
 
         """
         Args:
@@ -67,11 +74,13 @@ class Sorn(object):
             """ Generate weight matrix for E-E/ E-I connections with mean lamda incoming and 
                out-going connections per neuron """
 
-            weight_matrix = Initializer.generate_lambd_connections(synaptic_connection, Sorn.ne, Sorn.ni, lambd_w, lambd_std=1)
+            weight_matrix = Initializer.generate_lambd_connections(
+                synaptic_connection, Sorn.ne, Sorn.ni, lambd_w, lambd_std=1
+            )
 
         # Dense matrix for W_ie
 
-        elif (network_type == 'Dense') and (self_connection == 'False'):
+        elif (network_type == "Dense") and (self_connection == "False"):
 
             # Gaussian distribution of weights
             # weight_matrix = np.random.randn(Sorn.ne, Sorn.ni) + 2 # Small random values from gaussian distribution
@@ -134,9 +143,15 @@ class Plasticity(Sorn):
         super().__init__()
         self.nu = Sorn.nu  # Number of input units
         self.ne = Sorn.ne  # Number of excitatory units
-        self.eta_stdp = Sorn.eta_stdp  # STDP plasticity Learning rate constant; SORN1 and SORN2
-        self.eta_ip = Sorn.eta_ip  # Intrinsic plasticity learning rate constant; SORN1 and SORN2
-        self.eta_inhib = Sorn.eta_inhib  # Intrinsic plasticity learning rate constant; SORN2 only
+        self.eta_stdp = (
+            Sorn.eta_stdp
+        )  # STDP plasticity Learning rate constant; SORN1 and SORN2
+        self.eta_ip = (
+            Sorn.eta_ip
+        )  # Intrinsic plasticity learning rate constant; SORN1 and SORN2
+        self.eta_inhib = (
+            Sorn.eta_inhib
+        )  # Intrinsic plasticity learning rate constant; SORN2 only
         self.h_ip = 2 * Sorn.nu / Sorn.ne  # Target firing rate
         self.mu_ip = Sorn.mu_ip  # Mean target firing rate
         self.ni = int(0.2 * Sorn.ne)  # Number of inhibitory units in the network
@@ -157,9 +172,11 @@ class Plasticity(Sorn):
 
         for i in range(len(wee_t[0])):  # Each neuron i, Post-synaptic neuron
 
-            for j in range(len(wee_t[0:])):  # Incoming connection from jth pre-synaptic neuron to ith neuron
+            for j in range(
+                len(wee_t[0:])
+            ):  # Incoming connection from jth pre-synaptic neuron to ith neuron
 
-                if wee_t[j][i] != 0.:  # Check connectivity
+                if wee_t[j][i] != 0.0:  # Check connectivity
 
                     # Get the change in weight
                     delta_wee_t = self.eta_stdp * (xt[i] * xt_1[j] - xt_1[i] * xt[j])
@@ -220,14 +237,20 @@ class Plasticity(Sorn):
         # iSTDP applies only on the neurons which are connected.
         wei_t = wei.copy()
 
-        for i in range(len(wei_t[0])):  # Each neuron i, Post-synaptic neuron: means for each column;
+        for i in range(
+            len(wei_t[0])
+        ):  # Each neuron i, Post-synaptic neuron: means for each column;
 
-            for j in range(len(wei_t[0:])):  # Incoming connection from j, pre-synaptic neuron to ith neuron
+            for j in range(
+                len(wei_t[0:])
+            ):  # Incoming connection from j, pre-synaptic neuron to ith neuron
 
-                if wei_t[j][i] != 0.:  # Check connectivity
+                if wei_t[j][i] != 0.0:  # Check connectivity
 
                     # Get the change in weight
-                    delta_wei_t = - self.eta_inhib * yt_1[j] * (1 - xt[i] * (1 + 1 / self.mu_ip))
+                    delta_wei_t = (
+                        -self.eta_inhib * yt_1[j] * (1 - xt[i] * (1 + 1 / self.mu_ip))
+                    )
 
                     # Update the weight between jth neuron to i ""Different from notation in article
 
@@ -257,16 +280,15 @@ class Plasticity(Sorn):
             indexes = Initializer.get_unconnected_indexes(wee)
 
             # Choose any idx randomly such that i!=j
-            while True: 
+            while True:
                 idx_rand = random.choice(indexes)
                 if idx_rand[0] != idx_rand[1]:
-                    break  
+                    break
 
             wee[idx_rand[0]][idx_rand[1]] = 0.001
 
         return wee
 
-
     @staticmethod
     def initialize_plasticity():
 
@@ -275,15 +297,24 @@ class Plasticity(Sorn):
         # Create and initialize sorn object and variables
 
         sorn_init = Sorn()
-        WEE_init = sorn_init.initialize_weight_matrix(network_type=Sorn.network_type_ee, synaptic_connection='EE',
-                                                      self_connection='False',
-                                                      lambd_w=Sorn.lambda_ee)
-        WEI_init = sorn_init.initialize_weight_matrix(network_type=Sorn.network_type_ei, synaptic_connection='EI',
-                                                      self_connection='False',
-                                                      lambd_w=Sorn.lambda_ei)
-        WIE_init = sorn_init.initialize_weight_matrix(network_type=Sorn.network_type_ie, synaptic_connection='IE',
-                                                      self_connection='False',
-                                                      lambd_w=Sorn.lambda_ie)
+        WEE_init = sorn_init.initialize_weight_matrix(
+            network_type=Sorn.network_type_ee,
+            synaptic_connection="EE",
+            self_connection="False",
+            lambd_w=Sorn.lambda_ee,
+        )
+        WEI_init = sorn_init.initialize_weight_matrix(
+            network_type=Sorn.network_type_ei,
+            synaptic_connection="EI",
+            self_connection="False",
+            lambd_w=Sorn.lambda_ei,
+        )
+        WIE_init = sorn_init.initialize_weight_matrix(
+            network_type=Sorn.network_type_ie,
+            synaptic_connection="IE",
+            self_connection="False",
+            lambd_w=Sorn.lambda_ie,
+        )
 
         Wee_init = Initializer.zero_sum_incoming_check(WEE_init)
         # Wei_init = initializer.zero_sum_incoming_check(WEI_init.T)  # For SORN 1
@@ -294,9 +325,12 @@ class Plasticity(Sorn):
         v = np.count_nonzero(Wei_init)
         b = np.count_nonzero(Wie_init)
 
-        print('Network Initialized')
-        print('Number of connections in Wee %s , Wei %s, Wie %s' %(c, v, b))
-        print('Shapes Wee %s Wei %s Wie %s' % (Wee_init.shape, Wei_init.shape, Wie_init.shape))
+        print("Network Initialized")
+        print("Number of connections in Wee %s , Wei %s, Wie %s" % (c, v, b))
+        print(
+            "Shapes Wee %s Wei %s Wie %s"
+            % (Wee_init.shape, Wei_init.shape, Wie_init.shape)
+        )
 
         # Normalize the incoming weights
 
@@ -304,7 +338,9 @@ class Plasticity(Sorn):
         normalized_wei = Initializer.normalize_weight_matrix(Wei_init)
         normalized_wie = Initializer.normalize_weight_matrix(Wie_init)
 
-        te_init, ti_init = sorn_init.initialize_threshold_matrix(Sorn.te_min, Sorn.te_max, Sorn.ti_min, Sorn.ti_max)
+        te_init, ti_init = sorn_init.initialize_threshold_matrix(
+            Sorn.te_min, Sorn.te_max, Sorn.ti_min, Sorn.ti_max
+        )
         x_init, y_init = sorn_init.initialize_activity_vector(Sorn.ne, Sorn.ni)
 
         # Initializing variables from sorn_initialize.py
@@ -324,22 +360,25 @@ class Plasticity(Sorn):
 
         return NotImplementedError
 
+
 class MatrixCollection(Sorn):
     def __init__(self, phase, matrices=None):
         super().__init__()
 
         self.phase = phase
         self.matrices = matrices
-        if self.phase == 'plasticity' and self.matrices == None:
+        if self.phase == "plasticity" and self.matrices == None:
 
             self.time_steps = Sorn.time_steps + 1  # Total training steps
-            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = [0] * self.time_steps, [
-                0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                                 0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                          0] * self.time_steps, \
-                                                                             [0] * self.time_steps
+            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = (
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+            )
             wee, wei, wie, te, ti, x, y = Plasticity.initialize_plasticity()
 
             # Assign initial matrix to the master matrices
@@ -351,45 +390,49 @@ class MatrixCollection(Sorn):
             self.X[0] = x
             self.Y[0] = y
 
-        elif self.phase == 'plasticity' and self.matrices != None:
+        elif self.phase == "plasticity" and self.matrices != None:
 
             self.time_steps = Sorn.time_steps + 1  # Total training steps
-            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = [0] * self.time_steps, [
-                0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                                 0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                                 0] * self.time_steps, \
-                                                                             [0] * self.time_steps
+            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = (
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+            )
             # Assign matrices from plasticity phase to the new master matrices for training phase
-            self.Wee[0] = matrices['Wee']
-            self.Wei[0] = matrices['Wei']
-            self.Wie[0] = matrices['Wie']
-            self.Te[0] = matrices['Te']
-            self.Ti[0] = matrices['Ti']
-            self.X[0] = matrices['X']
-            self.Y[0] = matrices['Y']
+            self.Wee[0] = matrices["Wee"]
+            self.Wei[0] = matrices["Wei"]
+            self.Wie[0] = matrices["Wie"]
+            self.Te[0] = matrices["Te"]
+            self.Ti[0] = matrices["Ti"]
+            self.X[0] = matrices["X"]
+            self.Y[0] = matrices["Y"]
 
-        elif self.phase == 'training':
+        elif self.phase == "training":
 
             """NOTE:
             time_steps here is diferent for plasticity and training phase"""
             self.time_steps = Sorn.time_steps + 1  # Total training steps
-            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = [0] * self.time_steps, [
-                0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                                 0] * self.time_steps, \
-                                                                             [0] * self.time_steps, [
-                                                                                 0] * self.time_steps, \
-                                                                             [0] * self.time_steps
+            self.Wee, self.Wei, self.Wie, self.Te, self.Ti, self.X, self.Y = (
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+                [0] * self.time_steps,
+            )
             # Assign matrices from plasticity phase to new respective matrices for training phase
-            self.Wee[0] = matrices['Wee']
-            self.Wei[0] = matrices['Wei']
-            self.Wie[0] = matrices['Wie']
-            self.Te[0] = matrices['Te']
-            self.Ti[0] = matrices['Ti']
-            self.X[0] = matrices['X']
-            self.Y[0] = matrices['Y']
+            self.Wee[0] = matrices["Wee"]
+            self.Wei[0] = matrices["Wei"]
+            self.Wie[0] = matrices["Wie"]
+            self.Te[0] = matrices["Te"]
+            self.Ti[0] = matrices["Ti"]
+            self.X[0] = matrices["X"]
+            self.Y[0] = matrices["Y"]
 
     # @staticmethod
     def weight_matrix(self, wee, wei, wie, i):
@@ -423,20 +466,25 @@ class MatrixCollection(Sorn):
 
         return x_1, y_1
 
+
 class NetworkState(Plasticity):
-    
+
     """The evolution of network states"""
-    
+
     def __init__(self, v_t):
         super().__init__()
         self.v_t = v_t
         # Check the input feature size
-        
-        assert Sorn.nu==len(self.v_t),"Input units and input size mismatch: {} != {}".format(Sorn.nu,len(self.v_t))
-        if Sorn.nu!=Sorn.ne:
-            self.v_t = list(self.v_t) + [0.]*(Sorn.ne- Sorn.nu)        
+
+        assert Sorn.nu == len(
+            self.v_t
+        ), "Input units and input size mismatch: {} != {}".format(
+            Sorn.nu, len(self.v_t)
+        )
+        if Sorn.nu != Sorn.ne:
+            self.v_t = list(self.v_t) + [0.0] * (Sorn.ne - Sorn.nu)
         self.v_t = np.expand_dims(self.v_t, 1)
-    
+
     def incoming_drive(self, weights, activity_vector):
         # Broadcasting weight*acivity vectors
         incoming = weights * activity_vector
@@ -452,14 +500,26 @@ class NetworkState(Plasticity):
         yt = y[:, 1]
         yt = yt.reshape(self.ni, 1)
 
-        incoming_drive_e = np.expand_dims(self.incoming_drive(weights=wee, activity_vector=xt), 1)
-        incoming_drive_i = np.expand_dims(self.incoming_drive(weights=wei, activity_vector=yt), 1)
-        tot_incoming_drive = incoming_drive_e - incoming_drive_i + white_noise_e + np.asarray(self.v_t) - te
+        incoming_drive_e = np.expand_dims(
+            self.incoming_drive(weights=wee, activity_vector=xt), 1
+        )
+        incoming_drive_i = np.expand_dims(
+            self.incoming_drive(weights=wei, activity_vector=yt), 1
+        )
+        tot_incoming_drive = (
+            incoming_drive_e
+            - incoming_drive_i
+            + white_noise_e
+            + np.asarray(self.v_t)
+            - te
+        )
 
         """Heaviside step function"""
-        heaviside_step = np.expand_dims([0.] * len(tot_incoming_drive),1)
-        heaviside_step[tot_incoming_drive > 0] = 1.
-        xt_next = np.asarray(heaviside_step.copy())  # Additional Memory cost just for the sake of variable name
+        heaviside_step = np.expand_dims([0.0] * len(tot_incoming_drive), 1)
+        heaviside_step[tot_incoming_drive > 0] = 1.0
+        xt_next = np.asarray(
+            heaviside_step.copy()
+        )  # Additional Memory cost just for the sake of variable name
         return xt_next
 
     def inhibitory_network_state(self, wie, ti, x, white_noise_i):
@@ -469,13 +529,17 @@ class NetworkState(Plasticity):
         xt = x[:, 1]
         xt = xt.reshape(Sorn.ne, 1)
 
-        incoming_drive_e = np.expand_dims(self.incoming_drive(weights=wie, activity_vector=xt), 1)
+        incoming_drive_e = np.expand_dims(
+            self.incoming_drive(weights=wie, activity_vector=xt), 1
+        )
 
         tot_incoming_drive = incoming_drive_e + white_noise_i - ti
-        heaviside_step = np.expand_dims([0.] * len(tot_incoming_drive),1)
-        heaviside_step[tot_incoming_drive > 0] = 1.
+        heaviside_step = np.expand_dims([0.0] * len(tot_incoming_drive), 1)
+        heaviside_step[tot_incoming_drive > 0] = 1.0
 
-        yt_next = np.asarray(heaviside_step.copy())  # Additional Memory cost just for the sake of variable name
+        yt_next = np.asarray(
+            heaviside_step.copy()
+        )  # Additional Memory cost just for the sake of variable name
 
         return yt_next
 
@@ -488,45 +552,75 @@ class NetworkState(Plasticity):
         yt = y[:, 1]
         yt = yt.reshape(self.ni, 1)
 
-        incoming_drive_e = np.expand_dims(self.incoming_drive(weights=wee, activity_vector=xt), 1)
-        incoming_drive_i = np.expand_dims(self.incoming_drive(weights=wei, activity_vector=yt), 1)
+        incoming_drive_e = np.expand_dims(
+            self.incoming_drive(weights=wee, activity_vector=xt), 1
+        )
+        incoming_drive_i = np.expand_dims(
+            self.incoming_drive(weights=wei, activity_vector=yt), 1
+        )
 
         tot_incoming_drive = incoming_drive_e - incoming_drive_i + white_noise_e - te
 
-        heaviside_step = np.expand_dims([0.] * len(tot_incoming_drive),1)
-        heaviside_step[tot_incoming_drive > 0] = 1.
+        heaviside_step = np.expand_dims([0.0] * len(tot_incoming_drive), 1)
+        heaviside_step[tot_incoming_drive > 0] = 1.0
 
-        xt_next = np.asarray(heaviside_step.copy())  # Additional Memory cost just for the sake of variable name
+        xt_next = np.asarray(
+            heaviside_step.copy()
+        )  # Additional Memory cost just for the sake of variable name
 
         return xt_next
 
 
 # Simulate / Train SORN
 class Simulator_(Sorn):
-
     def __init__(self):
         super().__init__()
         pass
 
-    def simulate_sorn(self, inputs: np.array=None,  phase:str='plasticity', matrices: dict=None, time_steps:int=1,noise:bool=True, **kwargs):
-        
-        assert phase == 'plasticity' or 'training',"Phase can be either 'plasticity' or 'training'"
-        
+    def simulate_sorn(
+        self,
+        inputs: np.array = None,
+        phase: str = "plasticity",
+        matrices: dict = None,
+        time_steps: int = 1,
+        noise: bool = True,
+        **kwargs
+    ):
+
+        assert (
+            phase == "plasticity" or "training"
+        ), "Phase can be either 'plasticity' or 'training'"
+
         self.time_steps = time_steps
         Sorn.time_steps = time_steps
         self.phase = phase
         self.matrices = matrices
-        
-        kwargs_ = ['ne', 'nu', 'network_type_ee', 'network_type_ei', 'network_type_ie', 
-                     'lambda_ee','lambda_ei', 'lambda_ie', 
-                     'eta_stdp','eta_inhib', 'eta_ip', 
-                     'te_max', 'ti_max', 'ti_min', 'te_min', 'mu_ip','sigma_ip']
+
+        kwargs_ = [
+            "ne",
+            "nu",
+            "network_type_ee",
+            "network_type_ei",
+            "network_type_ie",
+            "lambda_ee",
+            "lambda_ei",
+            "lambda_ie",
+            "eta_stdp",
+            "eta_inhib",
+            "eta_ip",
+            "te_max",
+            "ti_max",
+            "ti_min",
+            "te_min",
+            "mu_ip",
+            "sigma_ip",
+        ]
         for key, value in kwargs.items():
             if key in kwargs_:
                 setattr(Sorn, key, value)
         # assert Sorn.nu == len(inputs[:,0]),"Size mismatch: Input != Nu "
-        Sorn.ni=int(0.2 * Sorn.ne)
-        
+        Sorn.ni = int(0.2 * Sorn.ne)
+
         # Initialize/Get the weight, threshold matrices and activity vectors
         matrix_collection = MatrixCollection(phase=self.phase, matrices=self.matrices)
 
@@ -541,21 +635,31 @@ class Simulator_(Sorn):
         # To get the last activation status of Exc and Inh neurons
         for i in tqdm.tqdm(range(self.time_steps)):
             """ Generate white noise"""
-            if noise:   
-                white_noise_e = Initializer.white_gaussian_noise(mu=0., sigma=0.04, t=Sorn.ne)
-                white_noise_i = Initializer.white_gaussian_noise(mu=0., sigma=0.04, t=Sorn.ni)
+            if noise:
+                white_noise_e = Initializer.white_gaussian_noise(
+                    mu=0.0, sigma=0.04, t=Sorn.ne
+                )
+                white_noise_i = Initializer.white_gaussian_noise(
+                    mu=0.0, sigma=0.04, t=Sorn.ni
+                )
             else:
-                white_noise_e, white_noise_i = 0.,0.
-               
-            network_state = NetworkState(inputs[:,i])  # Feed input and initialize network state
+                white_noise_e, white_noise_i = 0.0, 0.0
+
+            network_state = NetworkState(
+                inputs[:, i]
+            )  # Feed input and initialize network state
 
             # Buffers to get the resulting x and y vectors at the current time step and update the master matrix
             x_buffer, y_buffer = np.zeros((Sorn.ne, 2)), np.zeros((Sorn.ni, 2))
-            # TODO: Return te,ti values in next version # UNUSED 
+            # TODO: Return te,ti values in next version # UNUSED
             te_buffer, ti_buffer = np.zeros((Sorn.ne, 1)), np.zeros((Sorn.ni, 1))
 
             # Get the matrices and rename them for ease of reading
-            Wee, Wei, Wie = matrix_collection.Wee, matrix_collection.Wei, matrix_collection.Wie
+            Wee, Wei, Wie = (
+                matrix_collection.Wee,
+                matrix_collection.Wei,
+                matrix_collection.Wie,
+            )
             Te, Ti = matrix_collection.Te, matrix_collection.Ti
             X, Y = matrix_collection.X, matrix_collection.Y
 
@@ -563,17 +667,24 @@ class Simulator_(Sorn):
             frac_pos_active_conn.append((Wee[i] > 0.0).sum())
 
             """ Recurrent drive"""
-            r = network_state.recurrent_drive(Wee[i], Wei[i], Te[i], X[i], Y[i], white_noise_e)
+            r = network_state.recurrent_drive(
+                Wee[i], Wei[i], Te[i], X[i], Y[i], white_noise_e
+            )
 
             """Get excitatory states and inhibitory states given the weights and thresholds"""
             # x(t+1), y(t+1)
-            excitatory_state_xt_buffer = network_state.excitatory_network_state(Wee[i], Wei[i], Te[i], X[i], Y[i],
-                                                                                white_noise_e)
-            inhibitory_state_yt_buffer = network_state.inhibitory_network_state(Wie[i], Ti[i], X[i], white_noise_i)
+            excitatory_state_xt_buffer = network_state.excitatory_network_state(
+                Wee[i], Wei[i], Te[i], X[i], Y[i], white_noise_e
+            )
+            inhibitory_state_yt_buffer = network_state.inhibitory_network_state(
+                Wie[i], Ti[i], X[i], white_noise_i
+            )
 
             """ Update X and Y """
             x_buffer[:, 0] = X[i][:, 1]  # xt -->(becomes) xt_1
-            x_buffer[:, 1] = excitatory_state_xt_buffer.T  # New_activation; x_buffer --> xt
+            x_buffer[
+                :, 1
+            ] = excitatory_state_xt_buffer.T  # New_activation; x_buffer --> xt
 
             y_buffer[:, 0] = Y[i][:, 1]
             y_buffer[:, 1] = inhibitory_state_yt_buffer.T
@@ -594,7 +705,9 @@ class Simulator_(Sorn):
             Wee_t = plasticity.structural_plasticity(Wee_t)
 
             # iSTDP
-            Wei_t = plasticity.istdp(Wei[i], x_buffer, y_buffer, cutoff_weights=(0.0, 1.0))
+            Wei_t = plasticity.istdp(
+                Wei[i], x_buffer, y_buffer, cutoff_weights=(0.0, 1.0)
+            )
 
             # Synaptic scaling Wee
             Wee_t = Plasticity().ss(Wee_t)
@@ -611,28 +724,40 @@ class Simulator_(Sorn):
             Y_all[i] = y_buffer[:, 1]
             R_all[i] = r
 
-        plastic_matrices = {'Wee': matrix_collection.Wee[-1],
-                            'Wei': matrix_collection.Wei[-1],
-                            'Wie': matrix_collection.Wie[-1],
-                            'Te': matrix_collection.Te[-1], 'Ti': matrix_collection.Ti[-1],
-                            'X': X[-1], 'Y': Y[-1]}
+        plastic_matrices = {
+            "Wee": matrix_collection.Wee[-1],
+            "Wei": matrix_collection.Wei[-1],
+            "Wie": matrix_collection.Wie[-1],
+            "Te": matrix_collection.Te[-1],
+            "Ti": matrix_collection.Ti[-1],
+            "X": X[-1],
+            "Y": Y[-1],
+        }
 
         return plastic_matrices, X_all, Y_all, R_all, frac_pos_active_conn
 
+
 class Trainer_(Sorn):
-    
+
     """Args:
         inputs - one hot vector of inputs
     
         Returns:
         matrix_collection - collection of all weight matrices in dictionaries
         """
-            
+
     def __init__(self):
         super().__init__()
         pass
-    
-    def train_sorn(self, inputs: np.array=None, phase: str='plasticity', matrices: np.array=None, noise: bool=True,**kwargs):
+
+    def train_sorn(
+        self,
+        inputs: np.array = None,
+        phase: str = "plasticity",
+        matrices: np.array = None,
+        noise: bool = True,
+        **kwargs
+    ):
         """[summary]
 
         Args:
@@ -643,120 +768,158 @@ class Trainer_(Sorn):
 
         Returns:
             [type]: [description]
-        """       
-        assert phase == 'plasticity' or 'training',"Phase can be either 'plasticity' or 'training'"
-        
-        kwargs_ = ['ne', 'ni', 'network_type_ee', 'network_type_ei', 'network_type_ie', 
-                     'lambda_ee','lambda_ei', 'lambda_ie', 
-                     'eta_stdp','eta_inhib', 'eta_ip', 
-                     'te_max', 'ti_max', 'ti_min', 'te_min', 'mu_ip','sigma_ip']
+        """
+        assert (
+            phase == "plasticity" or "training"
+        ), "Phase can be either 'plasticity' or 'training'"
+
+        kwargs_ = [
+            "ne",
+            "ni",
+            "network_type_ee",
+            "network_type_ei",
+            "network_type_ie",
+            "lambda_ee",
+            "lambda_ei",
+            "lambda_ie",
+            "eta_stdp",
+            "eta_inhib",
+            "eta_ip",
+            "te_max",
+            "ti_max",
+            "ti_min",
+            "te_min",
+            "mu_ip",
+            "sigma_ip",
+        ]
         for key, value in kwargs.items():
             if key in kwargs_:
                 setattr(Sorn, key, value)
-        Sorn.ni=int(0.2 * Sorn.ne)
+        Sorn.ni = int(0.2 * Sorn.ne)
         # assert Sorn.nu == len(inputs[:,0]),"Size mismatch: Input != Nu "
-        
+
         self.phase = phase
         self.matrices = matrices
         self.time_steps = 1
         Sorn.time_steps = 1
         self.inputs = np.asarray(inputs)
-        
+
         # Collect the network activity at all time steps
-        X_all = [0]*self.time_steps
-        Y_all = [0]*self.time_steps
-        R_all = [0]*self.time_steps
-        
+        X_all = [0] * self.time_steps
+        Y_all = [0] * self.time_steps
+        R_all = [0] * self.time_steps
+
         frac_pos_active_conn = []
-         
-        matrix_collection = MatrixCollection(phase = self.phase, matrices = self.matrices)  
-        
+
+        matrix_collection = MatrixCollection(phase=self.phase, matrices=self.matrices)
+
         for i in range(1):
-            
+
             if noise:
-                white_noise_e = Initializer.white_gaussian_noise(mu= 0., sigma = 0.04,t = Sorn.ne)
-                white_noise_i = Initializer.white_gaussian_noise(mu= 0., sigma = 0.04,t = Sorn.ni)
+                white_noise_e = Initializer.white_gaussian_noise(
+                    mu=0.0, sigma=0.04, t=Sorn.ne
+                )
+                white_noise_i = Initializer.white_gaussian_noise(
+                    mu=0.0, sigma=0.04, t=Sorn.ni
+                )
             else:
-                white_noise_e = 0.
-                white_noise_i = 0.
-            
-            network_state = NetworkState(self.inputs)  # Feed Input as an argument to the class
-            
+                white_noise_e = 0.0
+                white_noise_i = 0.0
+
+            network_state = NetworkState(
+                self.inputs
+            )  # Feed Input as an argument to the class
+
             # Buffers to get the resulting x and y vectors at the current time step and update the master matrix
-            x_buffer, y_buffer = np.zeros(( Sorn.ne, 2)), np.zeros((Sorn.ni, 2))
+            x_buffer, y_buffer = np.zeros((Sorn.ne, 2)), np.zeros((Sorn.ni, 2))
             te_buffer, ti_buffer = np.zeros((Sorn.ne, 1)), np.zeros((Sorn.ni, 1))
 
             # Get the matrices and rename them for ease of reading
-            Wee, Wei, Wie = matrix_collection.Wee, matrix_collection.Wei, matrix_collection.Wie
+            Wee, Wei, Wie = (
+                matrix_collection.Wee,
+                matrix_collection.Wei,
+                matrix_collection.Wie,
+            )
             Te, Ti = matrix_collection.Te, matrix_collection.Ti
             X, Y = matrix_collection.X, matrix_collection.Y
-            
+
             """ Fraction of active connections between E-E network"""
             frac_pos_active_conn.append((Wee[i] > 0.0).sum())
-            
+
             # Recurrent drive at t+1 used to predict the next external stimuli
-            r = network_state.recurrent_drive(Wee[i], Wei[i], Te[i], X[i], Y[i],white_noise_e = white_noise_e)
+            r = network_state.recurrent_drive(
+                Wee[i], Wei[i], Te[i], X[i], Y[i], white_noise_e=white_noise_e
+            )
 
             """Get excitatory states and inhibitory states given the weights and thresholds"""
             # x(t+1), y(t+1)
-            excitatory_state_xt_buffer = network_state.excitatory_network_state(Wee[i], Wei[i], Te[i], X[i], Y[i],white_noise_e = white_noise_e)
-            inhibitory_state_yt_buffer = network_state.inhibitory_network_state(Wie[i], Ti[i], X[i],white_noise_i = white_noise_i)
-            
+            excitatory_state_xt_buffer = network_state.excitatory_network_state(
+                Wee[i], Wei[i], Te[i], X[i], Y[i], white_noise_e=white_noise_e
+            )
+            inhibitory_state_yt_buffer = network_state.inhibitory_network_state(
+                Wie[i], Ti[i], X[i], white_noise_i=white_noise_i
+            )
+
             """ Update X and Y """
             x_buffer[:, 0] = X[i][:, 1]  # xt -->xt_1
             x_buffer[:, 1] = excitatory_state_xt_buffer.T  # x_buffer --> xt
             y_buffer[:, 0] = Y[i][:, 1]
             y_buffer[:, 1] = inhibitory_state_yt_buffer.T
-            
-            if self.phase=='plasticity':
+
+            if self.phase == "plasticity":
                 plasticity = Plasticity()
 
-                # STDP 
-                Wee_t = plasticity.stdp(Wee[i],x_buffer,cutoff_weights = (0.0,1.0))
-                
+                # STDP
+                Wee_t = plasticity.stdp(Wee[i], x_buffer, cutoff_weights=(0.0, 1.0))
+
                 # Intrinsic plasticity
-                Te_t = plasticity.ip(Te[i],x_buffer)
-                
+                Te_t = plasticity.ip(Te[i], x_buffer)
+
                 # Structural plasticity
-                Wee_t = plasticity.structural_plasticity(Wee_t)      
-                
-                # iSTDP 
-                Wei_t = plasticity.istdp(Wei[i],x_buffer,y_buffer,cutoff_weights = (0.0,1.0))
-                
+                Wee_t = plasticity.structural_plasticity(Wee_t)
+
+                # iSTDP
+                Wei_t = plasticity.istdp(
+                    Wei[i], x_buffer, y_buffer, cutoff_weights=(0.0, 1.0)
+                )
+
                 # Synaptic scaling Wee
                 Wee_t = Plasticity().ss(Wee_t)
-                
+
                 # Synaptic scaling Wei
                 Wei_t = Plasticity().ss(Wei_t)
             else:
-                Wee_t, Wei_t, Te_t = Wee[i],Wei[i], Te[i]
-                
+                Wee_t, Wei_t, Te_t = Wee[i], Wei[i], Te[i]
+
             """Assign the matrices to the matrix collections"""
             matrix_collection.weight_matrix(Wee_t, Wei_t, Wie[i], i)
             matrix_collection.threshold_matrix(Te_t, Ti[i], i)
             matrix_collection.network_activity_t(x_buffer, y_buffer, i)
-            
-            X_all[i] = x_buffer[:,1]
-            Y_all[i] = y_buffer[:,1]
+
+            X_all[i] = x_buffer[:, 1]
+            Y_all[i] = y_buffer[:, 1]
             R_all[i] = r
-   
-        plastic_matrices = {'Wee':matrix_collection.Wee[-1], 
-                            'Wei': matrix_collection.Wei[-1], 
-                            'Wie':matrix_collection.Wie[-1],
-                            'Te': matrix_collection.Te[-1], 'Ti': matrix_collection.Ti[-1],
-                            'X': X[-1], 'Y': Y[-1]}
-        
-        return plastic_matrices,X_all,Y_all,R_all,frac_pos_active_conn
+
+        plastic_matrices = {
+            "Wee": matrix_collection.Wee[-1],
+            "Wei": matrix_collection.Wei[-1],
+            "Wie": matrix_collection.Wie[-1],
+            "Te": matrix_collection.Te[-1],
+            "Ti": matrix_collection.Ti[-1],
+            "X": X[-1],
+            "Y": Y[-1],
+        }
+
+        return plastic_matrices, X_all, Y_all, R_all, frac_pos_active_conn
 
 
 # global Trainer
 # global Simulator
 Trainer = Trainer_()
-Simulator = Simulator_()  
+Simulator = Simulator_()
 if __name__ == "__main__":
     pass
     # Instantiate Trainer and Simulator objects while import
 #    Trainer = Trainer_()
-#    Simulator = Simulator_()  
-
+#    Simulator = Simulator_()
 
